useDynLib(Epi, .registration=TRUE)
export(
  apc.frame,
  apc.fit,
  apc.plot,
  apc.lines,
 plot.apc,
lines.apc,
   pc.points,
   pc.lines,
   pc.matpoints,
   pc.matlines,
   pc.matshade,
   cp.points,
   cp.lines,
   cp.matpoints,
   cp.matlines,
   cp.matshade,
          cal.yr,
  as.Date.cal.yr,
  ccwc,
  ci.pd,
  ci.lin,
  ci.exp,
  ci.cum,
  ci.surv,
  ci.pred,
  ci.ratio,
  ci.mat,
  ci.Crisk,
  matshade,
  lls,
  clear,
  contr.orth,
  contr.2nd,
  contr.cum,
  contr.diff,
  detrend,
  decurve,
  dur,
  erl, surv1, surv2, erl1, yll,
  effx,
  effx.match,
  float,
  print.floated,
  gen.exp,
  mat2pol,
          Icens,
    print.Icens,
  summary.Icens,
  plotevent,
  fit.add,
  fit.mult,
  ftrend,
  fgrep,
  ngrep,
  lgrep,
  legendbox,
      plotCIF,
   stackedCIF,
 show.apc.LCa,
      apc.LCa,
          LCa.fit,
    print.LCa,
  summary.LCa,
  predict.LCa,
     plot.LCa,
  Lexis.diagram,
  Lexis.lines,
   Life.lines,
                Lexis,
              unLexis,
          merge.Lexis,
           plot.Lexis,
         points.Lexis,
          lines.Lexis,
         PY.ann.Lexis,
         subset.Lexis,
             "[.Lexis",
          cbind.Lexis,
          rbind.Lexis,
          order.Lexis,
           orderLexis,
            sortLexis,
          print.Lexis,
        summary.Lexis,
  print.summary.Lexis,
           splitLexis,
      transform.Lexis,
         levels.Lexis,
            nid.Lexis,
            bootLexis,
        Relevel.Lexis,
      factorize.Lexis,
             cutLexis,
            mcutLexis,
            rcutLexis,
           countLexis,
          stack.Lexis,
           tmat.Lexis,
          boxes.Lexis,
          boxes.matrix,
          boxes.MS,
         msdata.Lexis,
            etm.Lexis,
            crr.Lexis,
            glm.Lexis,
            gam.Lexis,
          coxph.Lexis,
            AaJ.Lexis,
             simLexis,
         addCov.Lexis,
        addDrug.Lexis,
         coarse.Lexis,
         subset.stacked.Lexis,
      transform.stacked.Lexis,
           plot.pState,
          lines.pState,
  Lexis2msm,
  entry,
  exit,
  status,
  timeBand,
  timeScales,
  timeSince,
  tsNA20,
  breaks,
  absorbing,
  transient,
  preceding,
  before,
  succeeding,
  after,
  tbox,
  dbox,
  fillarr,
  boxarr,
  boxes,
  factorize,
  rm.tr,
  PY.ann,
  N2Y,
  tmat,
  nState,
  pState,
  msdata,
        mh,
  print.mh,
  ncut,
  nice,
  NArray,
  ZArray,
  Ns,
  Termplot,
  pctab,
  plotEst,
  pointsEst,
  projection.ip,
  in.span,
  inSpan,
  id.span,
  idSpan,
  thinCol,
  linesEst,
  rateplot,
  Aplot,
  Pplot,
  Cplot,
  nid,
  Relevel,
  Relevel.factor,
  ROC,
  twoby2,
  Wald,
  stat.table,
  clogistic,
  poisreg,
  harm)
# Import generic methods
importFrom( utils, stack )
importFrom( splines, ns, bs )
importFrom( plyr, rbind.fill )
# importFrom( purrr, "%>%" )
importFrom( magrittr, "%>%" )
importFrom( dplyr, summarize, group_by, arrange, left_join, inner_join,
            select, rename, mutate, ungroup, filter, near)
importFrom( cmprsk, crr)
importFrom( etm, etm )
# importFrom( mstate, msdata )
importFrom( MASS, mvrnorm, ginv )
importFrom( survival, clogit, coxph, Surv, survfit )
importFrom( mgcv, gam )
importFrom( numDeriv, hessian )
importFrom( Matrix, nearPD )
importFrom( zoo, na.locf )
importFrom("grDevices", "gray", "rainbow", "adjustcolor", "dev.cur")
importFrom("graphics", "abline", "arrows", "axis", "box", "layout",
           "lines", "locator", "matlines", "matplot", "matpoints",
           "mtext", "par", "plot", "plot.new", "plot.window", "points",
           "polygon", "rect", "rug", "segments", "strheight",
           "strwidth", "text")
importFrom("stats", ".getXlevels", "AIC", "addmargins", "anova", "approx",
           "ave", "as.formula", "binomial", "coef", "complete.cases",
           "contr.sum", "dpois", "fisher.test", "fitted", "formula",
           "gaussian", "glm", "is.empty.model", "lm", "make.link",
           "median", "model.extract", "model.matrix", "model.offset",
           "model.response", "nlm", "pchisq", "pnorm", "poisson",
           "predict", "predict.lm", "qnorm", "qt", "quantile",
           "rpois", "runif", "termplot", "update", "vcov",
           "weighted.mean", "xtabs")
importFrom("utils", "flush.console", "str")

# register S3 methods
S3method(     print, Icens)
S3method(   summary, Icens)
S3method(     print, floated)
S3method(      plot, apc)
S3method(     lines, apc)
S3method(      plot, Lexis)
S3method(      plot, pState)
S3method(     lines, pState)
S3method(    points, Lexis)
S3method(     lines, Lexis)
S3method(    PY.ann, Lexis)
S3method(     merge, Lexis)
S3method(    subset, Lexis)
S3method(    subset, stacked.Lexis)
S3method(       "[", Lexis)
S3method(     cbind, Lexis)
S3method(     rbind, Lexis)
S3method(     print, Lexis)
S3method(   summary, Lexis)
S3method(     print, summary.Lexis)
S3method( transform, Lexis)
S3method( transform, stacked.Lexis)
S3method(    levels, Lexis)
S3method(       nid, Lexis)
S3method(       nid, default)
S3method(   Relevel, Lexis)
S3method(   Relevel, factor)
S3method(   Relevel, default)
S3method( factorize, Lexis)
S3method( factorize, default)
S3method(    addCov, Lexis)
S3method(   addDrug, Lexis)
S3method(     stack, Lexis)
S3method(      tmat, Lexis)
S3method(     boxes, Lexis)
S3method(     boxes, matrix)
S3method(     boxes, MS)
S3method(    msdata, Lexis)
S3method(       etm, Lexis)
S3method(       AaJ, Lexis)
S3method(     print, mh)
S3method(     print, LCa)
S3method(   summary, LCa)
S3method(   predict, LCa)
S3method(      plot, LCa)
S3method(     print, stat.table)
S3method(     print, clogistic)
S3method(      coef, clogistic)
S3method(      vcov, clogistic)
S3method(   as.Date, cal.yr)
